def stringify_main_options(options):
    main_options = ''
    for index, option in enumerate(options.keys()):
        main_options += f'{index + 1}. {option}\n'
    return main_options

def stringify_sub_options(options, main_option):
    sub_options = ''
    if main_option not in options.keys():
        return None

    for index, sub_option in enumerate(options[main_option]):
        sub_options += f'{index + 1}. {sub_option.name}\n'
    return sub_options
    
    




    


import sys
import os
import pytest
sys.path.append(os.path.dirname(sys.path[0]))
from src.wikisearch.wikisearcher import WikiSearcher

@pytest.fixture
def searcher():
    return WikiSearcher()

def test_method1(searcher):
    result = searcher.search('React.js')
    assert "React (also known as React.js or ReactJS) is a free and open-source front-end JavaScript library[3] for building user interfaces or UI components." in result

def test_method2(searcher):
    result = searcher.search('The Hills')
    assert type(result) == dict

def test_method3(searcher):
    result = searcher.search('The Hills')
    assert len(result.keys()) == 3

def test_method4(searcher):
    result = searcher.search('Spark')
    assert len(result.keys()) == 9

def test_method5(searcher):
    result = searcher.search('Python')
    assert type(result) == dict

def test_method6(searcher):
    result = searcher.search('Python')
    assert len(result.keys()) == 8

def test_method7(searcher):
    result = searcher.search('adasdasdasdasd')
    assert result == 'Invalid search. No result.'

def test_method8(searcher):
    result = searcher.search('2001')
    assert '2001 (MMI) was a common year starting on Monday' in result








from bs4.element import NavigableString, Tag
from pprint import pprint


def wiki_clean_option(body_content):
    p_empty_elt = body_content.find('p', class_='mw-empty-elt')
    table_content = body_content.find('div', class_='tocright')
    edits = body_content.find_all('span', class_='mw-editsection')
    short_desc = body_content.find('div', class_='shortdescription nomobile noexcerpt noprint searchaux')
    style = body_content.find('style')
    note = body_content.find('div', role='note')
    tables = body_content.find_all('table')
    redirect = body_content.find('span', class_='mw-redirectedfrom')
    elements_to_clean = [table_content, tables, edits, short_desc, style, note, redirect, p_empty_elt]
    for element in elements_to_clean:
        if isinstance(element, list):
            for el in element:
                el.decompose()
        else:
            if element is not None:
                element.decompose()
def wiki_clean_option_p(filtered_body_content):
    prev_element = None
    for i, element in enumerate(filtered_body_content):
        if element.name == 'h2':
            prev_element = element.name
        elif element.name == 'p' and prev_element != None:
            filtered_body_content.pop(i)
            continue
        else:
            prev_element = None


def wiki_clean_desc(body_content):
    short_desc = body_content.find('div', class_='shortdescription nomobile noexcerpt noprint searchaux')
    id_note = body_content.find('div', role='note')
    class_note = body_content.find('div', class_='hatnote navigation-not-searchable')
    p = body_content.find('p', class_='mw-empty-elt')
    tables = body_content.find_all('table')
    coordinates = body_content.find('span', id='coordinates')
    div_site_sub = body_content.find('div', id='siteSub')
    elements_to_clean = [short_desc, id_note, class_note, p, coordinates, div_site_sub, tables ]

    for element in elements_to_clean:
        if element is not None:
            element.decompose()
        elif isinstance(element, list):
            for el in element:
                if el is not None:
                    el.decompose()

def wiki_clean_desc_recursive(body_content):
    for child in body_content.children:
        if child is None or (child.name == 'p' and len(child.attrs) == 0) or isinstance(child, NavigableString):
            pass
        else:
            child.decompose()

def has_link(tag):
    if not isinstance(tag, Tag):
        raise NameError('Invalid tag.')
    a_tag = tag.find('a')
    return a_tag





from bs4 import BeautifulSoup
from more_itertools import peekable
from .article import ArticleLink
from .wikiutil.wikiutil import *

import requests

class WikiSearcher:

    def __init__(self):            
        self.__URL_ROOT = 'https://en.wikipedia.org'
        self.__URL_SEARCH = 'https://en.wikipedia.org/wiki'
    
    def search(self, search):
        search_url = ''
        if isinstance(search, ArticleLink):
            search_url = self.__URL_ROOT + search.link
        else:
            search_replaced = search.replace(' ', '_')
            search_url = self.__URL_SEARCH + f'/{search_replaced}'
        raw_html = requests.get(search_url).text
        soup = BeautifulSoup(raw_html, 'lxml')
        body_content = soup.find('div', class_='mw-parser-output')
        if body_content is None:
            return 'Invalid search. No result.'
        if 'most often refers to:' not in soup.text and 'may also refer to:' not in soup.text and 'may refer to:' not in soup.text:
            wiki_clean_desc_recursive(body_content)
            return body_content.p.text
        else:
            wiki_clean_option(body_content)
            body_children = []
            for child in body_content.children:
                if child.name is not None and child.name not in ['style', 'h3', 'h4']:
                    body_children.append(child)
            wiki_clean_option_p(body_children)
            peekable_children = peekable(body_children)
            options = {}
            prev_key = None
            for child in peekable_children:
                peek_child = None
                try:
                    peek_child = peekable_children.peek()
                except StopIteration:
                    pass
                if child.name == 'p' and peek_child.name == 'h2':
                    continue
                if child.name == 'p' or child.name == 'h2':
                    prev_key = child.text.strip()
                    a_tag = child.find('a')
                    if a_tag is not None:
                        prev_key = f'{prev_key} (already a description select for more info.)'
                        options[prev_key] = []
                        article_link = ArticleLink(a_tag.attrs['title'], a_tag.attrs['href'])
                        options[prev_key].append(article_link)
                    else:
                        if peek_child.name  in ['p', 'h2']:
                            prev_key = f'{prev_key} (already a description)'
                            options[prev_key] = None


                else:
                    options[prev_key] = []
                    ul_tags = []
                    ul_tags.append(child)
                    try:
                        while peek_child is not None and peek_child.name == 'ul':
                            ul_tags.append(next(peekable_children))
                            peek_child = peekable_children.peek()
                        
                    except StopIteration:
                        pass
                    li_tags = []
                    for ul in ul_tags:
                        lis = ul.find_all('li', recursive=False)
                        if lis is not None:
                            li_tags.extend(lis)
                    for li in li_tags:
                        a_tag = has_link(li)
                        if a_tag is None:
                            options[prev_key].append(li.text + ' (no other link)')
                        else:
                            article_link = ArticleLink(a_tag.attrs['title'], a_tag.attrs['href'])
                            options[prev_key].append(article_link)
            
            return options
                            

                        
                              
                
                
            
            
            
            

        
        



class ArticleLink:
    def __init__(self, name, link):
        self.__name = name
        self.__link = link

    @property
    def name(self):
        return self.__name
    @property
    def link(self):
        return self.__link

    def __str__(self):
        return self.__link
